#include <hip/hip_runtime.h>

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

#include "hip_launch.h"

float GetRandomNumber(float min, float max) {
  static const double fraction = 1.0 / (RAND_MAX + 1.0);
  return min + static_cast<float>((max - min + 1) * (std::rand() * fraction));
}

void Malloc(void** ptr, size_t size) {
  if (size <= 0) {
    assert(false);
  }

  auto err = hipMalloc(ptr, size);
  assert(err == hipSuccess);
}

int main(int argc, char** argv) {
  hipStream_t stream;
  auto err = hipStreamCreateWithFlags(&stream, hipStreamDefault);
  assert(err == hipSuccess);

  int N = 32;
  float input_vector[N];
  for (int i = 0; i < N; i++) {
    input_vector[i] = GetRandomNumber(0, 10);
  }

  for (int i = 0; i < N; i++) {
    std::cout << input_vector[i] << " ";
    if (i == 255) std::cout << std::endl;
  }
  std::cout << std::endl;

  float *d_in, *d_out;
  Malloc((void**)&d_in, N * sizeof(float));
  Malloc((void**)&d_out, N * sizeof(float));

  err = hipMemcpy(d_in, input_vector, N * sizeof(float), hipMemcpyHostToDevice);
  assert(err == hipSuccess);

  hipModule_t module;
  err = hipModuleLoad(&module, "Kernel.bin");
  assert(err == hipSuccess);

  hipFunction_t func;
  err = hipModuleGetFunction(&func, module, "add");
  assert(err == hipSuccess);

  float alpha = 10;

  std::vector<void*> args = {&d_in, &d_out, &alpha, &N};

  constexpr int LOCAL_SIZE = 256;
  dim3 gws((N + LOCAL_SIZE - 1) / LOCAL_SIZE * LOCAL_SIZE);
  dim3 lws(LOCAL_SIZE);

  ExtLaunchKernel(func, gws, lws, 0, stream, args.data());

  float output_vector[N];

  err =
      hipMemcpy(output_vector, d_out, N * sizeof(float), hipMemcpyDeviceToHost);
  std::cout << "===\n";
  for (int i = 0; i < N; i++) {
    std::cout << output_vector[i] << " ";
    if (i == 255) std::cout << std::endl;
  }
  std::cout << std::endl;

  return 0;
}
